package cz.fidentis.analyst.mesh.KdTree;

import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.mesh.core.MeshPoint;

import java.util.HashMap;
import java.util.Map;


public class KdNode {
    private final int depth;
    private Map<MeshFacet, Integer> facets = new HashMap<>();

    private KdNode lesser = null;
    private KdNode greater = null;
    private KdNode parent;

    public KdNode(MeshFacet facet,int index, int depth, KdNode parent) {
        this.facets.putIfAbsent(facet, index);
        this.depth = depth;

        this.parent = parent;

    }

    public void addFacet(MeshFacet facet, int index){
        this.facets.putIfAbsent(facet, index);
    }

    //return only one index (index of MeshPoint in the first MeshFacet)
    public int getIndex() {
        Map.Entry<MeshFacet, Integer> entry = facets.entrySet().iterator().next();
        return entry.getValue();
    }

    public int getDepth() {
        return depth;
    }

    public MeshPoint getId() {
        Map.Entry<MeshFacet, Integer> entry = facets.entrySet().iterator().next();
        return entry.getKey().getVertex(entry.getValue());
    }

    public KdNode getLesser() {
        return lesser;
    }

    public KdNode getGreater() {
        return greater;
    }

   public KdNode getParent() {
        return parent;
    }


    public KdNode setLesser(KdNode lesser) {
        this.lesser = lesser;
        return this;
    }

    public KdNode setGreater(KdNode greater) {
        this.greater = greater;
        return this;
    }

    public Map<MeshFacet, Integer> getFacets() {
        return facets;
    }

    public KdNode setParent(KdNode parent) {
        this.parent = parent;
        return this;
    }

}
