package cz.fidentis.analyst.mesh.KdTree;
import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.mesh.core.MeshFacetImpl;
import cz.fidentis.analyst.mesh.core.MeshPoint;
import cz.fidentis.analyst.mesh.core.MeshPointImpl;
import org.junit.jupiter.api.Test;

import javax.vecmath.Vector3d;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import static org.junit.jupiter.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.assertTrue;




public class kdTreeTest {

    private Vector3d position = new Vector3d(0.1f, 0.5f, 0.7f);
    private Vector3d normalAndTextCoord = new Vector3d(0,0,0);


    @Test
    public void testPut(){
        MeshPoint p = new MeshPointImpl(position,normalAndTextCoord,normalAndTextCoord);

       // List<MeshPoint> points = new LinkedList<>();
        List<MeshFacet> facets = new LinkedList<>();
       // points.add(p);
        MeshFacet facet = new MeshFacetImpl();
        facet.addVertex(p);
        facets.add(facet);
        KdTreeImpl tree = new KdTreeImpl(facets);

        assertTrue(tree.containPoint(p));
    }

    @Test
    public void testPutNothing(){
        MeshPoint p = new MeshPointImpl(position,normalAndTextCoord,normalAndTextCoord);

      //  List<MeshPoint> points = new LinkedList<>();
        List<MeshFacet> facets2 = new LinkedList<>();
        MeshFacet facet = new MeshFacetImpl();
        KdTreeImpl tree = new KdTreeImpl(facets2);

        assertFalse(tree.containPoint(p));
    }

    public double distancePoints(MeshPoint point1, MeshPoint point2) {
        return Math.sqrt(Math.pow(point1.getPosition().x - point2.getPosition().x, 2)
                + Math.pow(point1.getPosition().y - point2.getPosition().y, 2)
                + Math.pow(point1.getPosition().z - point2.getPosition().z, 2));
    }


    private MeshPoint findClosestSequential(List<MeshPoint> points, MeshPoint p){
        if(points.isEmpty()){
            return null;
        }

        MeshPoint closest = points.get(0);
        double dis = distancePoints(closest, p);

        for(MeshPoint cl : points){
            if(distancePoints(cl, p) < dis){
                dis = distancePoints(cl, p);
                closest = cl;
            }
        }

        return closest;
    }

    @Test
    public void testFindClosestAlreadyIn(){
        List<MeshFacet> facets = new LinkedList<>();

        MeshFacet facet = new MeshFacetImpl();


        List<MeshPoint> points = new LinkedList<>();
        Vector3d positionOfPoints;

        for(int i = 0; i < 10; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        facets.add(facet);
        KdTreeImpl tree = new KdTreeImpl(facets);

        Random r = new Random();

        MeshPoint p = points.get(r.nextInt(points.size()));
        MeshPoint found = tree.nearestNeighbour(p);

        assertEquals(found, p);
        assertEquals(0.0f, distancePoints(found, p));
    }

    @Test
    public void testFindClosestAlreadyInTwoFacets(){
        List<MeshFacet> facets = new LinkedList<>();

        MeshFacet facet1 = new MeshFacetImpl();
        MeshFacet facet2 = new MeshFacetImpl();


        List<MeshPoint> points = new LinkedList<>();
        Vector3d positionOfPoints;

        for(int i = 0; i < 5; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet1.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        for(int i = 5; i < 10; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet2.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        facets.add(facet1);
        facets.add(facet2);
        KdTreeImpl tree = new KdTreeImpl(facets);

        Random r = new Random();

        MeshPoint p = points.get(r.nextInt(points.size()));
        MeshPoint found = tree.nearestNeighbour(p);

        assertEquals(found, p);
        assertEquals(0.0f, distancePoints(found, p));
    }

    @Test
    public void testCountOfFacets(){
        List<MeshFacet> facets = new LinkedList<>();

        MeshFacet facet1 = new MeshFacetImpl();
        MeshFacet facet2 = new MeshFacetImpl();


        List<MeshPoint> points = new LinkedList<>();
        Vector3d positionOfPoints;

        for(int i = 0; i < 10; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet1.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        for(int i = 0; i < 10; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet2.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        facets.add(facet1);
        facets.add(facet2);
        KdTreeImpl tree = new KdTreeImpl(facets);

        Random r = new Random();

        MeshPoint p = points.get(r.nextInt(points.size()));
        MeshPoint found = tree.nearestNeighbour(p);

        assertEquals(found, p);
        assertEquals(2,tree.nearestNeighbourCountOfFacets(p));
    }

    @Test
    public void testFindClosestNotInTreeTwoFacets(){
        List<MeshFacet> facets = new LinkedList<>();

        MeshFacet facet1 = new MeshFacetImpl();
        MeshFacet facet2 = new MeshFacetImpl();


        List<MeshPoint> points = new LinkedList<>();
        Vector3d positionOfPoints;

        for(int i = 0; i < 5; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet1.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        for(int i =5; i < 10; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet2.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }


        facets.add(facet1);
        facets.add(facet2);
        KdTreeImpl tree = new KdTreeImpl(facets);

        MeshPoint p = new MeshPointImpl(new Vector3d(2.5f, 0.3f, 1.2f), normalAndTextCoord,normalAndTextCoord);
        MeshPoint found = tree.nearestNeighbour(p);

        MeshPoint seq = findClosestSequential(points, p);

        assertEquals(found, seq);
        assertEquals(distancePoints(found, p), distancePoints(seq, p));
    }
    @Test
    public void testIndexFoundTwoFacetsNoDuplicates(){
        List<MeshFacet> facets = new LinkedList<>();

        MeshFacet facet1 = new MeshFacetImpl();
        MeshFacet facet2 = new MeshFacetImpl();


        List<MeshPoint> points = new LinkedList<>();
        Vector3d positionOfPoints;

        for(int i = 5; i < 20; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet1.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        for(int i =0; i < 5; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
            facet2.addVertex(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }


        facets.add(facet1);
        facets.add(facet2);
        KdTreeImpl tree = new KdTreeImpl(facets);

        Random r = new Random();

        MeshPoint p = points.get(r.nextInt(points.size()));
        int found = tree.nearestNeighbourIndex(p);

        assertEquals(points.get(found), p);
        assertEquals(0.0f, distancePoints(points.get(found), p));
    }

/*
    @Test
    public void testIndexFound(){
        List<MeshPoint> points = new LinkedList<>();
        Vector3d positionOfPoints;

        for(int i = 5; i < 10; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }
        for(int i = 0; i < 5; i++){
            positionOfPoints = new Vector3d(0.1f * i, 0.5f * i, 0.7f * i);
            points.add(new MeshPointImpl(positionOfPoints, normalAndTextCoord, normalAndTextCoord));
        }

        KdTreeImpl tree = new KdTreeImpl(points);
        Random r = new Random();

        MeshPoint p = points.get(r.nextInt(points.size()));
        int found = tree.nearestNeighbourIndex(p);

        assertEquals(points.get(found), p);
        assertEquals(0.0f, distancePoints(points.get(found), p));
    }*/

}


